@@ -255,38 +255,6 @@ M_TimeOperation(void (*threadFunc)(void *),
     return SECSuccess;
 }
 
-#define GFP_POPULATE(params, name_v)                         \
-    params.name = name_v;                                    \
-    if ((params.name < ECCurve_noName) ||                    \
-        (params.name > ECCurve_pastLastCurve))               \
-        goto cleanup;                                        \
-    params.type = ec_params_named;                           \
-    params.curveOID.data = NULL;                             \
-    params.curveOID.len = 0;                                 \
-    params.curve.seed.data = NULL;                           \
-    params.curve.seed.len = 0;                               \
-    params.DEREncoding.data = NULL;                          \
-    params.DEREncoding.len = 0;                              \
-    params.arena = NULL;                                     \
-    params.fieldID.size = ecCurve_map[name_v]->size;         \
-    params.fieldID.type = ec_field_GFp;                      \
-    hexString2SECItem(params.arena, &params.fieldID.u.prime, \
-                      ecCurve_map[name_v]->irr);             \
-    hexString2SECItem(params.arena, &params.curve.a,         \
-                      ecCurve_map[name_v]->curvea);          \
-    hexString2SECItem(params.arena, &params.curve.b,         \
-                      ecCurve_map[name_v]->curveb);          \
-    genenc[0] = '0';                                         \
-    genenc[1] = '4';                                         \
-    genenc[2] = '\0';                                        \
-    strcat(genenc, ecCurve_map[name_v]->genx);               \
-    strcat(genenc, ecCurve_map[name_v]->geny);               \
-    hexString2SECItem(params.arena, &params.base,            \
-                      genenc);                               \
-    hexString2SECItem(params.arena, &params.order,           \
-                      ecCurve_map[name_v]->order);           \
-    params.cofactor = ecCurve_map[name_v]->cofactor;
-
 /* Test curve using specific field arithmetic. */
 #define ECTEST_NAMED_GFP(name_c, name_v)                               \
     if (usefreebl) {                                                   \
@@ -317,6 +285,9 @@ hexString2SECItem(PLArenaPool *arena, SECItem *item, const char *str)
     int byteval = 0;
     int tmp = PORT_Strlen(str);
 
+    PORT_Assert(arena);
+    PORT_Assert(item);
+
     if ((tmp % 2) != 0) {
         return NULL;
     }
@@ -327,20 +298,22 @@ hexString2SECItem(PLArenaPool *arena, SECItem *item, const char *str)
         tmp -= 2;
     }
 
-    item->data = (unsigned char *)PORT_Alloc(tmp / 2);
-    if (item->data == NULL)
+    item = SECITEM_AllocItem(arena, item, tmp / 2);
+    if (item == NULL) {
         return NULL;
-    item->len = tmp / 2;
+    }
 
     while (str[i]) {
-        if ((str[i] >= '0') && (str[i] <= '9'))
+        if ((str[i] >= '0') && (str[i] <= '9')) {
             tmp = str[i] - '0';
-        else if ((str[i] >= 'a') && (str[i] <= 'f'))
+        } else if ((str[i] >= 'a') && (str[i] <= 'f')) {
             tmp = str[i] - 'a' + 10;
-        else if ((str[i] >= 'A') && (str[i] <= 'F'))
+        } else if ((str[i] >= 'A') && (str[i] <= 'F')) {
             tmp = str[i] - 'A' + 10;
-        else
+        } else {
+            /* item is in arena and gets freed by the caller */
             return NULL;
+        }
 
         byteval = byteval * 16 + tmp;
         if ((i % 2) != 0) {
@@ -579,11 +552,7 @@ ECDH_DeriveWrap(ECPrivateKey *priv, ECPublicKey *pub, int *dummy)
 
     rv = ECDH_Derive(&pub->publicValue, &pub->ecParams,
                      &priv->privateValue, 0, &secret);
-#ifdef notdef
-    if (rv == SECSuccess) {
-        PORT_Free(secret.data);
-    }
-#endif
+    SECITEM_FreeItem(&secret, PR_FALSE);
     return rv;
 }
 
@@ -593,7 +562,7 @@ ECDH_DeriveWrap(ECPrivateKey *priv, ECPublicKey *pub, int *dummy)
 SECStatus
 ectest_curve_freebl(ECCurveName curve, int iterations, int numThreads)
 {
-    ECParams ecParams;
+    ECParams ecParams = { 0 };
     ECPrivateKey *ecPriv = NULL;
     ECPublicKey ecPub;
     SECItem sig;
@@ -603,8 +572,40 @@ ectest_curve_freebl(ECCurveName curve, int iterations, int numThreads)
     double signRate, deriveRate;
     char genenc[3 + 2 * 2 * MAX_ECKEY_LEN];
     SECStatus rv = SECFailure;
+    PLArenaPool *arena;
 
-    GFP_POPULATE(ecParams, curve);
+    arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
+    if (!arena) {
+        return SECFailure;
+    }
+
+    if ((curve < ECCurve_noName) || (curve > ECCurve_pastLastCurve)) {
+        PORT_FreeArena(arena, PR_FALSE);
+        return SECFailure;
+    }
+
+    ecParams.name = curve;
+    ecParams.type = ec_params_named;
+    ecParams.curveOID.data = NULL;
+    ecParams.curveOID.len = 0;
+    ecParams.curve.seed.data = NULL;
+    ecParams.curve.seed.len = 0;
+    ecParams.DEREncoding.data = NULL;
+    ecParams.DEREncoding.len = 0;
+
+    ecParams.fieldID.size = ecCurve_map[curve]->size;
+    ecParams.fieldID.type = ec_field_GFp;
+    hexString2SECItem(arena, &ecParams.fieldID.u.prime, ecCurve_map[curve]->irr);
+    hexString2SECItem(arena, &ecParams.curve.a, ecCurve_map[curve]->curvea);
+    hexString2SECItem(arena, &ecParams.curve.b, ecCurve_map[curve]->curveb);
+    genenc[0] = '0';
+    genenc[1] = '4';
+    genenc[2] = '\0';
+    strcat(genenc, ecCurve_map[curve]->genx);
+    strcat(genenc, ecCurve_map[curve]->geny);
+    hexString2SECItem(arena, &ecParams.base, genenc);
+    hexString2SECItem(arena, &ecParams.order, ecCurve_map[curve]->order);
+    ecParams.cofactor = ecCurve_map[curve]->cofactor;
 
     PORT_Memset(digestData, 0xa5, sizeof(digestData));
     digest.data = digestData;
@@ -640,6 +641,8 @@ ectest_curve_freebl(ECCurveName curve, int iterations, int numThreads)
     }
 
 cleanup:
+    PORT_FreeArena(arena, PR_FALSE);
+    PORT_FreeArena(ecPriv->ecParams.arena, PR_FALSE);
     return rv;
 }
 
@@ -762,6 +765,8 @@ main(int argv, char **argc)
 #endif
 
 cleanup:
+    rv |= NSS_Shutdown();
+
     if (rv != SECSuccess) {
         printf("Error: exiting with error value\n");
     }
