@@ -27,7 +27,7 @@ static PRDescIdentity test_fd_identity = PR_INVALID_IO_LAYER;
   PR_ASSERT(PR_FALSE);                           \
   PR_SetError(PR_NOT_IMPLEMENTED_ERROR, 0)
 
-#define LOG(a) std::cerr << name_ << ": " << a << std::endl;
+#define LOG(a) std::cerr << name_ << ": " << a << std::endl
 
 class Packet : public DataBuffer {
  public:
@@ -47,7 +47,10 @@ class Packet : public DataBuffer {
 
 // Implementation of NSPR methods
 static PRStatus DummyClose(PRFileDesc *f) {
+  DummyPrSocket *io = reinterpret_cast<DummyPrSocket *>(f->secret);
   f->secret = nullptr;
+  f->dtor(f);
+  delete io;
   return PR_SUCCESS;
 }
 
@@ -256,7 +259,10 @@ DummyPrSocket::~DummyPrSocket() {
 
 void DummyPrSocket::Reset() {
   delete filter_;
-  peer_ = nullptr;
+  if (peer_) {
+    peer_->SetPeer(nullptr);
+    peer_ = nullptr;
+  }
   while (!input_.empty())
   {
     Packet* front = input_.front();
@@ -393,6 +399,15 @@ void Poller::Shutdown() {
   instance = nullptr;
 }
 
+Poller::~Poller()
+{
+  while (!timers_.empty()) {
+    Timer *timer = timers_.top();
+    timers_.pop();
+    delete timer;
+  }
+}
+
 void Poller::Wait(Event event, DummyPrSocket *adapter, PollTarget *target,
                   PollCallback cb) {
   auto it = waiters_.find(adapter);
