@@ -114,14 +114,14 @@ bool TlsAgent::ConfigServerCert(const std::string& name, bool updateKeyBits) {
   return rv == SECSuccess;
 }
 
-bool TlsAgent::EnsureTlsSetup() {
+bool TlsAgent::EnsureTlsSetup(PRFileDesc *modelSocket) {
   // Don't set up twice
   if (ssl_fd_) return true;
 
   if (adapter_->mode() == STREAM) {
-    ssl_fd_ = SSL_ImportFD(nullptr, pr_fd_);
+    ssl_fd_ = SSL_ImportFD(modelSocket, pr_fd_);
   } else {
-    ssl_fd_ = DTLS_ImportFD(nullptr, pr_fd_);
+    ssl_fd_ = DTLS_ImportFD(modelSocket, pr_fd_);
   }
 
   EXPECT_NE(nullptr, ssl_fd_);
@@ -132,6 +132,11 @@ bool TlsAgent::EnsureTlsSetup() {
   EXPECT_EQ(SECSuccess, rv);
   if (rv != SECSuccess) return false;
 
+  // Needs to be set before configuring server certs.
+  rv = SSL_OptionSet(ssl_fd_, SSL_NO_STEP_DOWN, PR_TRUE);
+  EXPECT_EQ(SECSuccess, rv);
+  if (rv != SECSuccess) return false;
+
   if (role_ == SERVER) {
     EXPECT_TRUE(ConfigServerCert(name_, true));
 
@@ -182,6 +187,8 @@ SECStatus TlsAgent::GetClientAuthDataHook(void* self, PRFileDesc* fd,
                                           CERTCertificate** cert,
                                           SECKEYPrivateKey** privKey) {
   TlsAgent* agent = reinterpret_cast<TlsAgent*>(self);
+  ScopedCERTCertificate peerCert(SSL_PeerCertificate(agent->ssl_fd()));
+  EXPECT_TRUE(peerCert) << "Client should be able to see the server cert";
   if (agent->GetClientAuthCredentials(cert, privKey)) {
     return SECSuccess;
   }
@@ -205,8 +212,8 @@ void TlsAgent::RequestClientAuth(bool requireAuth) {
   expect_client_auth_ = true;
 }
 
-void TlsAgent::StartConnect() {
-  EXPECT_TRUE(EnsureTlsSetup());
+void TlsAgent::StartConnect(PRFileDesc *model) {
+  EXPECT_TRUE(EnsureTlsSetup(model));
 
   SECStatus rv;
   rv = SSL_ResetHandshake(ssl_fd_, role_ == SERVER ? PR_TRUE : PR_FALSE);
@@ -232,7 +239,7 @@ void TlsAgent::EnableCiphersByKeyExchange(SSLKEAType kea) {
     ASSERT_EQ(SECSuccess, rv);
     EXPECT_EQ(sizeof(csinfo), csinfo.length);
 
-    if (csinfo.keaType == kea) {
+    if (csinfo.keaType == kea && !csinfo.isExportable) {
       rv = SSL_CipherPrefSet(ssl_fd_, SSL_ImplementedCiphers[i], PR_TRUE);
       EXPECT_EQ(SECSuccess, rv);
     }
@@ -249,7 +256,7 @@ void TlsAgent::EnableCiphersByAuthType(SSLAuthType authType) {
                                           &csinfo, sizeof(csinfo));
     ASSERT_EQ(SECSuccess, rv);
 
-    if (csinfo.authType == authType) {
+    if (csinfo.authType == authType && !csinfo.isExportable) {
       rv = SSL_CipherPrefSet(ssl_fd_, SSL_ImplementedCiphers[i], PR_TRUE);
       EXPECT_EQ(SECSuccess, rv);
     }
@@ -278,6 +285,14 @@ void TlsAgent::SetSessionCacheEnabled(bool en) {
   EXPECT_EQ(SECSuccess, rv);
 }
 
+void TlsAgent::Set0RttEnabled(bool en) {
+  EXPECT_TRUE(EnsureTlsSetup());
+
+  SECStatus rv = SSL_OptionSet(ssl_fd_, SSL_ENABLE_0RTT_DATA,
+                               en ? PR_TRUE : PR_FALSE);
+  EXPECT_EQ(SECSuccess, rv);
+}
+
 void TlsAgent::SetVersionRange(uint16_t minver, uint16_t maxver) {
    vrange_.min = minver;
    vrange_.max = maxver;
@@ -346,25 +361,28 @@ void TlsAgent::SetSignatureAlgorithms(const SSLSignatureAndHashAlg* algorithms,
   EXPECT_EQ(i, configuredCount) << "algorithms in use were all set";
 }
 
-void TlsAgent::CheckKEAType(SSLKEAType type) const {
+void TlsAgent::CheckKEA(SSLKEAType type, size_t kea_size) const {
   EXPECT_EQ(STATE_CONNECTED, state_);
   EXPECT_EQ(type, csinfo_.keaType);
+  EXPECT_EQ(kea_size, info_.keaKeyBits);
+}
 
+void TlsAgent::CheckKEA(SSLKEAType type) const {
   PRUint32 ecKEAKeyBits = SSLInt_DetermineKEABits(server_key_bits_,
                                                   csinfo_.authType);
-
   switch (type) {
-      case ssl_kea_ecdh:
-          EXPECT_EQ(ecKEAKeyBits, info_.keaKeyBits);
-          break;
-      case ssl_kea_dh:
-          EXPECT_EQ(2048U, info_.keaKeyBits);
-          break;
-      case ssl_kea_rsa:
-          EXPECT_EQ(server_key_bits_, info_.keaKeyBits);
-          break;
-      default:
-          break;
+    case ssl_kea_ecdh:
+      CheckKEA(type, ecKEAKeyBits);
+      break;
+    case ssl_kea_dh:
+      CheckKEA(type, 2048);
+      break;
+    case ssl_kea_rsa:
+      CheckKEA(type, server_key_bits_);
+      break;
+    default:
+      EXPECT_TRUE(false) << "Unknown KEA type";
+      break;
   }
 }
 
@@ -436,7 +454,12 @@ void TlsAgent::CheckAlpn(SSLNextProtoState expected_state,
                                   &chosen_len, sizeof(chosen));
   EXPECT_EQ(SECSuccess, rv);
   EXPECT_EQ(expected_state, state);
-  EXPECT_EQ(expected, std::string(chosen, chosen_len));
+  if (state == SSL_NEXT_PROTO_NO_SUPPORT) {
+    EXPECT_EQ("", expected);
+  } else {
+    EXPECT_NE("", expected);
+    EXPECT_EQ(expected, std::string(chosen, chosen_len));
+  }
 }
 
 void TlsAgent::EnableSrtp() {
@@ -526,8 +549,11 @@ void TlsAgent::Connected() {
 
   if (expected_version_ >= SSL_LIBRARY_VERSION_TLS_1_3) {
     PRInt32 cipherSuites = SSLInt_CountTls13CipherSpecs(ssl_fd_);
-    EXPECT_EQ(((mode_ == DGRAM) && (role_ == CLIENT)) ? 2 : 1, cipherSuites);
+    // We use one ciphersuite in each direction, plus one that's kept around
+    // by DTLS for retransmission.
+    EXPECT_EQ(((mode_ == DGRAM) && (role_ == CLIENT)) ? 3 : 2, cipherSuites);
   }
+
   SetState(STATE_CONNECTED);
 }
 
@@ -549,6 +575,18 @@ void TlsAgent::CheckExtendedMasterSecret(bool expected) {
       << "unexpected extended master secret state for " << name_;
 }
 
+void TlsAgent::CheckEarlyDataAccepted(bool expected) {
+  if (version() < SSL_LIBRARY_VERSION_TLS_1_3) {
+    expected = false;
+  }
+  ASSERT_EQ(expected, info_.earlyDataAccepted != PR_FALSE)
+      << "unexpected early data state for " << name_;
+}
+
+void TlsAgent::CheckSecretsDestroyed() {
+  ASSERT_EQ(PR_TRUE, SSLInt_CheckSecretsDestroyed(ssl_fd_));
+}
+
 void TlsAgent::DisableRollbackDetection() {
   ASSERT_TRUE(EnsureTlsSetup());
 
