@@ -22,9 +22,11 @@ namespace nss_test {
 const uint8_t kTlsChangeCipherSpecType = 20;
 const uint8_t kTlsAlertType = 21;
 const uint8_t kTlsHandshakeType = 22;
+const uint8_t kTlsApplicationDataType = 23;
 
 const uint8_t kTlsHandshakeClientHello = 1;
 const uint8_t kTlsHandshakeServerHello = 2;
+const uint8_t kTlsHandshakeEncryptedExtensions = 8;
 const uint8_t kTlsHandshakeCertificate = 11;
 const uint8_t kTlsHandshakeServerKeyExchange = 12;
 const uint8_t kTlsHandshakeCertificateVerify = 15;
@@ -67,6 +69,13 @@ inline uint16_t NormalizeTlsVersion(uint16_t version) {
   return version;
 }
 
+inline uint16_t TlsVersionToDtlsVersion(uint16_t version ) {
+  if (version == 0x0302) {
+    return 0xfeff;
+  }
+  return 0xffff - version + 0x0201;
+}
+
 inline void WriteVariable(DataBuffer* target, size_t index,
                           const DataBuffer& buf, size_t len_size) {
   target->Write(index, static_cast<uint32_t>(buf.len()), len_size);
